use std::{ops::Deref, sync::Arc};

use aws_config::{BehaviorVersion, Region};
use aws_sdk_s3::config::Credentials;

use self::infra::aws_s3::s3_client;

#[derive(Clone)]
pub struct SharedApp(Arc<App>);

impl Deref for SharedApp {
    type Target = Arc<App>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl SharedApp {
    pub fn new(app: App) -> Self {
        Self(Arc::new(app))
    }
}

pub struct App {
    pub s3_client: aws_sdk_s3::Client,
}

impl App {
    pub async fn new() -> anyhow::Result<Self> {
        Ok(Self {
            s3_client: s3_client().await?,
        })
    }
}

mod infra {
    pub mod aws_s3 {
        use aws_config::{BehaviorVersion, Region};
        use aws_sdk_s3::config::Credentials;

        pub async fn s3_client() -> anyhow::Result<aws_sdk_s3::Client> {
            let shared_config = aws_config::defaults(BehaviorVersion::latest())
                .region(Region::new("eu-west-1"))
                .credentials_provider(Credentials::new(
                    std::env::var("AWS_ACCESS_KEY_ID")?,
                    std::env::var("AWS_SECRET_ACCESS_KEY")?,
                    None,
                    None,
                    "flux_releaser",
                ));

            let config = aws_sdk_s3::config::Builder::from(&shared_config.load().await)
                .endpoint_url(std::env::var("AWS_ENDPOINT_URL")?)
                .build();

            let client = aws_sdk_s3::Client::from_conf(config);

            let buckets = client.list_buckets().send().await?;

            buckets
                .buckets
                .iter()
                .flatten()
                .flat_map(|b| &b.name)
                .for_each(|n| tracing::debug!("test: found aws bucket: {}", n));

            Ok(client)
        }
    }
}
